#!/usr/bin/python
# coding=utf8

"""
    
"""

__import__('pkg_resources').declare_namespace(__name__)
__version__ = '0.0.1'
__author__ = 'hk'

import sys
import logging
from logging import Logger
from logging import Handler, LogRecord

CRITICAL = 50
FATAL = CRITICAL
ERROR = 40
WARNING = 30
WARN = WARNING
INFO = 20
DEBUG = 10
NOTSET = 0

try:
    unicode
    _unicode = True
except NameError:
    _unicode = False


class MyLocal(object):

    def __init__(self, **kwargs):
        self.__dict__.update(**kwargs)


class KokologHandler(Handler, object):

    def __init__(self):
        self._name = ''
        Handler.__init__(self)

    def emit(self, record):
        pass


class KokologRecord(LogRecord):
    def __init__(self, name, level, fn, lno, msg, args, kwargs, exc_info, func=None):
        self.kwargs = kwargs
        super(KokologRecord, self).__init__(name, level, fn, lno, msg, args, exc_info, func)


class KokologLogger(Logger):
    def debug(self, msg, *args, **kwargs):
        if self.isEnabledFor(DEBUG):
            self._log(DEBUG, msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        if self.isEnabledFor(INFO):
            self._log(INFO, msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        if self.isEnabledFor(WARNING):
            self._log(WARNING, msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        if self.isEnabledFor(ERROR):
            self._log(ERROR, msg, *args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        if self.isEnabledFor(CRITICAL):
            self._log(CRITICAL, msg, *args, **kwargs)

    def log(self, level, msg, *args, **kwargs):
        if not isinstance(level, int):
            if raiseExceptions:
                raise TypeError("level must be an integer")
            else:
                return
        if self.isEnabledFor(level):
            self._log(level, msg, *args, **kwargs)


    def _log(self, level, msg, *args, **kwargs):
        if logging._srcfile:
            try:
                fn, lno, func = self.findCaller()
            except ValueError:
                fn, lno, func = "(unknown file)", 0, "(unknown function)"
        else:
            fn, lno, func = "(unknown file)", 0, "(unknown function)"
        exc_info = kwargs.get('exc_info')
        extra = kwargs.get('extra')
        if exc_info:
            if not isinstance(exc_info, tuple):
                exc_info = sys.exc_info()
        record = self.makeRecord(self.name, level, fn, lno, msg, args, kwargs, exc_info, func, extra)
        self.handle(record)

    def makeRecord(self, name, level, fn, lno, msg, args, kwargs, exc_info, func=None, extra=None):
        addition = [one for one in kwargs.keys() if not one in ('exc_info', 'extra', 'printlevel')]
        if not '%s' in msg:
            msg = msg + ' '.join(['%s' for one in args])
        for one in addition:
            msg = msg + '\n' + one + ':  %s'
        args = tuple([str(one) for one in args] + [str(kwargs[one]) for one in addition])
        rv = KokologRecord(name, level, fn, lno, msg, args, kwargs, exc_info, func)
        if extra is not None:
            for key in extra:
                if (key in ["message", "asctime"]) or (key in rv.__dict__):
                    raise KeyError("Attempt to overwrite %r in LogRecord" % key)
                rv.__dict__[key] = extra[key]
        return rv

Logger.manager.setLoggerClass(KokologLogger)


#!/usr/bin/python
# coding=utf-8

"""
    
"""

import time
import collections

def Enum(**enums):
    return type('Enum', (), enums)

def unicode2utf8(obj):
    if isinstance(obj, unicode):
        return obj.encode('utf8')
    if isinstance(obj, str):
        return obj
    elif isinstance(obj, collections.Mapping):
        return dict(map(unicode2utf8, obj.iteritems()))
    elif isinstance(obj, collections.Iterable):
        return type(obj)(map(unicode2utf8, obj))
    else:
        return obj

def _cs(obj, encoding='utf8'):
    if isinstance(obj, unicode):
        return obj.encode(encoding)
    elif isinstance(obj, str):
        return obj
    else:
        return str(obj)

def _cu(string, encoding='utf8'):
    if isinstance(string, unicode):
        return string
    elif isinstance(string, str):
        try:
            return string.decode(encoding)
        except:
            import chardet
            det = chardet.detect(string)
            if det['encoding']:
                return string.decode(det['encoding'], 'ignore')
            else:
                return string.decode('gbk', 'ignore')
    else:
        return unicode(string)

def strQ2B(ustring):
    rstring = ""
    if isinstance(ustring, str):
        ustring = _cu(ustring)
    for uchar in ustring:
        inside_code=ord(uchar)
        if inside_code==0x3000:
            inside_code=0x0020
        else:
            inside_code-=0xfee0
        if inside_code<0x0020 or inside_code>0x7e:
            rstring += uchar
        else:
            rstring += unichr(inside_code)
    return rstring

def isdtiformat(dtistr, dtifmt):
    dtistr = dtistr or ''
    dtifmt = dtifmt or ''
    try:
        time.strptime(dtistr, dtifmt)
        return True
    except:
        return False

if __name__ == '__main__':
    pass



#!/usr/bin/python
# coding=utf8

"""
    安装包工具
"""

import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()

requires = [
    ]

kokolog = __import__('kokolog')
setup(name='kokolog',
version=kokolog.__version__,
description='wecatch kokolog',
author='haokuan',
author_email='jingdaohao@gmail.com',
url='https://github.com/listen-lavender/kokolog',
keywords='wecatch > ',
packages=find_packages(),
install_requires=requires,
scripts=[],
)




#!/usr/bin/python
# coding=utf-8

"""
   traceback message or other files
"""
import sys
import os
import datetime
import logging
import traceback
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler

from . import MyLocal
from character import _cs, _cu

CFG = MyLocal(debug=True, dir=os.path.expanduser('~/log/'), handlers=[], exist=[])


def _print(*args):
    """
        Print txt by coding GBK.

        *args
          list, list of printing contents
        
    """
    if not CFG.debug:
        return
    if not args:
        return
    encoding = 'gbk'
    args = [_cs(a, encoding) for a in args]
    f_back = None
    try:
        raise Exception
    except:
        f_back = sys.exc_traceback.tb_frame.f_back
    f_name = f_back.f_code.co_name
    filename = os.path.basename(f_back.f_code.co_filename)
    m_name = os.path.splitext(filename)[0]
    prefix = ('[%s.%s]'%(m_name, f_name)).ljust(20, ' ')
    if os.name == 'nt':
        for i in range(len(args)):
            v = args [i]
            if isinstance(v, str):
                args[i] = v #v.decode('utf8').encode('gbk')
            elif isinstance(v, unicode):
                args[i] = v.encode('gbk')
    print '[%s]'%str(datetime.datetime.now()), prefix, ' '.join(args)


def _print_err(*args):
    """
        Print errors.

        *args
          list, list of printing contents
        
    """
    if not CFG.debug:
        return
    if not args:
        return
    encoding = 'utf8' if os.name == 'posix' else 'gbk'
    args = [_cs(a, encoding) for a in args]
    f_back = None
    try:
        raise Exception
    except:
        f_back = sys.exc_traceback.tb_frame.f_back
    f_name = f_back.f_code.co_name
    filename = os.path.basename(f_back.f_code.co_filename)
    m_name = os.path.splitext(filename)[0]
    prefix = ('[%s.%s]'%(m_name, f_name)).ljust(20, ' ')
    print bcolors.FAIL+'[%s]'%str(datetime.datetime.now()), prefix, ' '.join(args) + bcolors.ENDC


def logprint(logname, category, level='INFO', backupCount=15):
    """
        Print logs by datetime.

        logname
          string, file name
        category
          string, category path of logs file in log directory
        level
          string, restrict whether logs to be printed or not
        backupCount
          int, how many backups can be reserved
        
    """
    path = os.path.join(CFG.dir, category.strip('/'), logname.strip('/') + '.log')
    print "log path:", path
    if not os.path.exists(path[:path.rindex('/')]):
        os.makedirs(path[:path.rindex('/')])
    # Initialize logger
    logger = logging.getLogger(logname)
    frt = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    hdr = logging.StreamHandler(sys.stdout)
    hdr.setFormatter(frt)
    hdr._name = '##_sh_##'
    if not hdr._name in CFG.exist:
        logger.addHandler(hdr)
        CFG.exist.append(hdr._name)

    hdr = TimedRotatingFileHandler(path, 'D', 1, backupCount)
    hdr.setFormatter(frt)
    hdr._name = '##_rfh_##'
    if not hdr._name in CFG.exist:
        logger.addHandler(hdr)
        CFG.exist.append(hdr._name)


    if level.upper() == 'NOTEST':
        level == logging.NOTSET
    elif level.upper() == 'DEBUG':
        level == logging.DEBUG
    elif level.upper() == 'WARNING':
        level == logging.WARNING
    elif level.upper() == 'ERROR':
        level == logging.ERROR
    elif level.upper() == 'CRITICAL':
        level == logging.CRITICAL
    else:
        level == logging.INFO
    logger.setLevel(level)

    def _wraper(*args, **kwargs):
        if not CFG.debug:
            return
        if not args:
            return
            
        for hdr in CFG.handlers:
            if not hdr._name in CFG.exist:
                logger.addHandler(hdr)
                CFG.exist.append(hdr._name)

        encoding = 'utf8' if os.name == 'posix' else 'gbk'
        args = [_cu(a, encoding) for a in args]

        prefix = ''

        pl = kwargs.get('printlevel', 'info').upper()
        if pl == 'DEBUG':
            try:
                logger.debug('', *args, **kwargs)
            except:
                t, v, b = sys.exc_info()
                err_messages = traceback.format_exception(t, v, b)
                print 'Error: %s' % ','.join(err_messages)
        elif pl == 'WARNING':
            try:
                logger.warning('', *args, **kwargs)
            except:
                t, v, b = sys.exc_info()
                err_messages = traceback.format_exception(t, v, b)
                print 'Error: %s' % ','.join(err_messages)
        elif pl == 'ERROR':
            try:
                logger.error('', *args, **kwargs)
            except:
                t, v, b = sys.exc_info()
                err_messages = traceback.format_exception(t, v, b)
                print 'Error: %s' % ','.join(err_messages)
        elif pl == 'CRITICAL':
            try:
                logger.critical('', *args, **kwargs)
            except:
                t, v, b = sys.exc_info()
                err_messages = traceback.format_exception(t, v, b)
                print 'Error: %s' % ','.join(err_messages)
        else:
            try:
                logger.info('', *args, **kwargs)
            except:
                t, v, b = sys.exc_info()
                err_messages = traceback.format_exception(t, v, b)
                print 'Error: %s' % ','.join(err_messages)
    return _wraper, logger


def fileprint(filename, category, level=logging.DEBUG, maxBytes=1024*10124*100,
             backupCount=0):
    """
        Print files by file size.

        filename
          string, file name
        category
          string, category path of logs file in log directory
        level
          enumerated type of logging module, restrict whether logs to be printed or not
        maxBytes
          int, max limit of file size
        backupCount
          int, allowed numbers of file copys
        
    """
    path = os.path.join(CFG.filedir, category, filename)

    # Initialize filer
    filer = logging.getLogger(filename)
    frt = logging.Formatter('%(message)s')

    hdr = RotatingFileHandler(path, 'a', maxBytes, backupCount, 'utf-8')
    hdr.setFormatter(frt)
    hdr._name = '##_rfh_##'
    already_in = False
    for _hdr in filer.handlers:
        if _hdr._name == '##_rfh_##':
            already_in = True
            break
    if not already_in:
        filer.addHandler(hdr)

    hdr = logging.StreamHandler(sys.stdout)
    hdr.setFormatter(frt)
    hdr._name = '##_sh_##'
    already_in = False
    for _hdr in filer.handlers:
        if _hdr._name == '##_sh_##':
            already_in = True
    if not already_in:
        filer.addHandler(hdr)

    filer.setLevel(level)
    def _wraper(*args):
        if not args:
            return
        encoding = 'utf8' if os.name == 'posix' else 'gbk'
        args = [_cu(a, encoding) for a in args]
        filer.info(' '.join(args))
    return _wraper, filer

class bcolors:
    """
        定义常用颜色
    """
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'

if __name__ == '__main__':
    pass



#!/usr/bin/python
# coding=utf-8

"""
    tools of handling files
"""

import os

def modulepath(filename):
    """
    Find the relative path to its module of a python file if existing.

    filename
      string, name of a python file
    """
    filepath = os.path.abspath(filename)
    prepath = filepath[:filepath.rindex('/')]
    postpath = '/'
    if prepath.count('/') == 0 or not os.path.exists(prepath + '/__init__.py'):
        flag = False
    else:
        flag = True
    while True:
        if prepath.endswith('/lib') or prepath.endswith('/bin') or prepath.endswith('/site-packages'):
            break
        elif flag and (prepath.count('/') == 0 or not os.path.exists(prepath + '/__init__.py')):
            break
        else:
            for f in os.listdir(prepath):
                if '.py' in f:
                    break
            else:
                break
            postpath = prepath[prepath.rindex('/'):].split('-')[0].split('_')[0] + postpath
            prepath = prepath[:prepath.rindex('/')]
    return postpath.lstrip('/') + filename.split('/')[-1].replace('.pyc', '').replace('.py', '') + '/'

def modulename(filename):
    """
    Find the modulename from filename.

    filename
      string, name of a python file
    """
    return filename.split('/')[-1].replace('.pyc', '').replace('.py', '')

if __name__ == '__main__':
    print modulepath(__file__)
